@@ -26,51 +26,29 @@
 #include "fill.h"
 
 typedef struct {
-  int used;
-  Coord coord;
-} Coord_entry;
-
-typedef struct {
-  Coord_entry *start;
-  Coord_entry *ptr;
+  Coord *start;
   int entries, bufmax;
-  int curr_entry;
 } Startbuf;
 
-static void add_start (Coord, Startbuf *);
+static void push_start (Coord, Startbuf *);
 static void track_starts (int *, Startbuf *, enum fill_op (Coord),
 			  Coord);
+static Coord pop_start (Startbuf *);
 
-/* TODO: This function is ugly and those pointers suck!  Clean this
-   mess, please.  */
 static void
-add_start (loc, list)
+push_start (loc, list)
      Coord loc;
      Startbuf *list;
 {
-  Coord_entry *new_entry;
-
-  if (list->entries++ == list->bufmax)
+  if (list->entries == list->bufmax)
     {
-      if (list->entries % CHUNK_SIZE == 0)
-	{
-	  int ptr_diff;
-	  ptr_diff = (int) ((list->ptr - list->start)
-			    * sizeof (Coord_entry));
-	  list->start = (Coord_entry *) realloc (list->start,
-						 (list->entries /
-						  CHUNK_SIZE + 1));
-	  list->ptr = (Coord_entry *) ((int) list->start + ptr_diff);
-	}
-      new_entry = (Coord_entry *) ((int) list->start + list->bufmax++
-				   * sizeof (Coord_entry));
+      list->bufmax++;
+      if (list->entries % CHUNK_SIZE == CHUNK_SIZE-1)
+	list->start = (Coord *) realloc (list->start, (list->bufmax +
+						       CHUNK_SIZE) *
+					 sizeof (Coord));
     }
-  else
-    {
-      for (new_entry = list->start; new_entry->used; new_entry++);
-    }
-  new_entry->used = 1;
-  new_entry->coord = loc;
+  list->start[list->entries++] = loc;
 }
 
 static void
@@ -90,11 +68,19 @@ track_starts (tracking, list, check, loc)
       if (check (loc))
 	{
 	  *tracking = 1;
-	  add_start (loc, list);
+	  push_start (loc, list);
 	}
     }
 }
 
+/* Return the last entry on the list.  Decrease list size.  */
+static Coord
+pop_start (list)
+     Startbuf *list;
+{
+  return list->start[--list->entries];
+}
+
 extern enum fill_op
 fill (loc, check, mark)
      Coord loc;
@@ -112,20 +98,18 @@ fill (loc, check, mark)
   /* There's got to be at least one "left" entry.  */
   left.entries = left.bufmax = 1;
   right.entries = right.bufmax = 0;
-  left.curr_entry = right.curr_entry = 0;
 
-  left.ptr = left.start = (Coord_entry *) my_malloc
-    (CHUNK_SIZE * sizeof (Coord_entry));
-  right.ptr = right.start = (Coord_entry *) my_malloc
-    (CHUNK_SIZE * sizeof (Coord_entry));
-  left.start->used = 1;
-  left.start->coord = loc;
+  left.start = (Coord *) my_malloc (CHUNK_SIZE * sizeof (Coord));
+  right.start = (Coord *) my_malloc (CHUNK_SIZE * sizeof (Coord));
+  *left.start = loc;
 
   while (1)
     {
       Startbuf *now_dir, *opp_dir;
       int up_start, down_start;
       enum fill_op check_status;
+      int last_dir = 0;
+      Coord location;
 
       /* Array left_scan has the surrounding cells in order:	632
 	 Array right_scan has the same order, mirrored.		8.1
@@ -145,51 +129,46 @@ fill (loc, check, mark)
 	  scan = left_scan;
 	  now_dir = &left;
 	  opp_dir = &right;
+	  location = pop_start (&left);
 	}
       else if (right.entries)
 	{
 	  scan = right_scan;
 	  now_dir = &right;
 	  opp_dir = &left;
+	  location = pop_start (&right);
 	}
       else
 	break;
 
-      check_status = check (now_dir->ptr->coord);
-      if (check_status == BREAK)
-	{
-	  ret_status = BREAK;
-	  break;
-	}
-      if (check_status)
+      check_status = check (location);
+      if (check_status == SUCCESSFUL)
 	{
 	  Coord opp_start;
-	  opp_start = add_coord (now_dir->ptr->coord, *scan++);
+	  opp_start = add_coord (location, *scan++);
 	  if (check (opp_start))
-	    add_start (opp_start, opp_dir);
+	    push_start (opp_start, opp_dir);
 	  
 	  /* Do initial checks for vertical search.  */
 	  track_starts (&up_start, now_dir, check, add_coord
-			(now_dir->ptr->coord, *scan++));
+			(location, *scan++));
 	  track_starts (&up_start, now_dir, check, add_coord
-			(now_dir->ptr->coord, *scan++));
+			(location, *scan++));
 	  track_starts (&down_start, now_dir, check, add_coord
-			(now_dir->ptr->coord, *scan++));
+			(location, *scan++));
 	  track_starts (&down_start, now_dir, check, add_coord
-			(now_dir->ptr->coord, *scan++));
+			(location, *scan++));
 
 	  do
 	    {
-	      if (mark (now_dir->ptr->coord) == SUCCESSFUL
-		  && ret_status == 0)
+	      if (mark (location) == SUCCESSFUL && ret_status == 0)
 		ret_status = 1;
 	      track_starts (&up_start, now_dir, check, add_coord
-			    (now_dir->ptr->coord, scan[0]));
+			    (location, scan[0]));
 	      track_starts (&down_start, now_dir, check, add_coord
-			    (now_dir->ptr->coord, scan[1]));
-	      now_dir->ptr->coord = add_coord (now_dir->ptr->coord,
-					       scan[2]);
-	      check_status = check (now_dir->ptr->coord);
+			    (location, scan[1]));
+	      location = add_coord (location, scan[2]);
+	      check_status = check (location);
 	    }
 	  while (check_status == SUCCESSFUL);
 	}
@@ -198,16 +177,6 @@ fill (loc, check, mark)
 	  ret_status = BREAK;
 	  break;
 	}
-      /* Free the entry.  */
-      now_dir->ptr->used = 0;
-      now_dir->entries--;
-      /* Search the next entry.  */
-      if (now_dir->entries > 0)
-	{
-	  now_dir->ptr = now_dir->start;
-	  while (now_dir->ptr->used == 0)
-	    now_dir->ptr++;
-	}
     }
   /* Free the memory used by the start tables.  */
   free (left.start);
