@@ -175,6 +175,7 @@ load_map (map_dir)
   fclose (file);
   game_map.bit_cross >>= 1;
   set_modes ();
+  game_map.sel_mode = game_map.def_mode;
   return (char *) NULL;
 }
 
@@ -282,6 +283,15 @@ set_map_bit (pos, status)
   game_map.map[parse_coord (pos)] |= status;
 }
 
+extern void
+clear_map_bit (status)
+     map_val status;
+{
+  register int i;
+  for (i = 0; i < game_map.size; i++)
+    game_map.map[i] &= status;
+}
+
 static enum cross_act
 check_surrounding (loc, me)
      Coord loc;
@@ -402,7 +412,8 @@ toggle_cross (Coord loc)
       if (cross_type == ANY_CROSS)
 	set_modes ();
       else
-	game_map.modes = game_map.def_mode = MODE_GUE;
+	game_map.modes = game_map.def_mode = game_map.sel_mode =
+	  MODE_GUE;
       return 1;
     }
   return 0;
@@ -421,7 +432,7 @@ set_modes ()
       if (game_map.n_cross == 0)
 	{
 	  game_map.modes = MODE_ATT | MODE_DEF | MODE_GUE;
-	  game_map.def_mode = MODE_ATT;
+	  game_map.def_mode = MODE_DEF;
 	}
       else if (game_map.n_cross > 3)
 	{
@@ -442,6 +453,7 @@ set_modes ()
 		    {
 		      game_map.modes = MODE_ATT;
 		      game_map.def_mode = MODE_ATT;
+		      game_map.sel_mode = game_map.def_mode;
 		      return;
 		    }
 		}
@@ -470,11 +482,14 @@ set_modes ()
 		      game_map.def_mode = MODE_ATT;
 		    }
 		  /* Stop search.  */
-		  return;
+		  break;
 		}
 	    }
 	}
     }
+  /* If selected mode becomes impossible, revert to default action.  */
+  if (!(game_map.sel_mode & game_map.modes))
+    game_map.sel_mode = game_map.def_mode;
 }
 
 static int
@@ -518,7 +533,8 @@ dist_to_capital (player, loc)
 extern char
 need_update ()
 {
-  /* Need not to be thread-safe.  */
+  /* Need not to be thread-safe.  As if I'm ever going to make motti
+     multi-threaded...  */
   static char last_modes, last_attacks, last_turn;
   register char update = 0;
 
@@ -561,3 +577,45 @@ need_update ()
   last_modes = game_map.modes;
   return update;
 }
+
+/* I choose to not implement a 'select_prev' function.  */
+extern void
+select_next ()
+{
+  switch (game_map.sel_mode)
+    {
+    case MODE_ATT:
+      if (game_map.modes & ~MODE_ATT)
+	{
+	  if (game_map.modes & MODE_DEF)
+	    game_map.sel_mode = MODE_DEF;
+	  else
+	    game_map.sel_mode = MODE_GUE;
+	}
+      break;
+    case MODE_DEF:
+      if (game_map.modes & ~MODE_DEF)
+	{
+	  if (game_map.modes & MODE_GUE)
+	    game_map.sel_mode = MODE_GUE;
+	  else
+	    game_map.sel_mode = MODE_ATT;
+	}
+      break;
+      /* If no attack available, then it's quaranteed there's no
+	 defend.  */
+    case MODE_GUE:
+      if (game_map.modes & MODE_ATT)
+	game_map.sel_mode = MODE_ATT;
+      break;
+    }
+}
+
+extern void
+clear_crosses ()
+{
+  game_map.n_cross = 0;
+  game_map.bit_cross = 0;
+  clear_map_bit ((map_val) ~MASK_CROSS);
+  set_modes ();
+}
