@@ -32,7 +32,7 @@
 #define UPDATE_TURN	8
 
 /* A hell will propably break loose if this is changed.  */
-#define MAXPLAYERS 2
+#define MAXPLAYERS 31
 
 typedef struct
 {
@@ -81,6 +81,9 @@ struct map_struct
   /* Default mode of action.  Based on the number of crosses, their
      placement, and the most sensible action with those crosses.  */
   unsigned char def_mode;
+  /* Selected mode of action.  Changed to default mode of action, if
+     becomes impossible.  */
+  unsigned char sel_mode;
   /* Pointer to the array of the map.  */
   map_val *map;
 };
@@ -107,9 +110,12 @@ extern Coord add_coord (Coord, Coord);
 extern int cmp_coord (Coord, Coord);
 extern void set_map (Coord, map_val);
 extern void set_map_bit (Coord, map_val);
+extern void clear_map_bit (map_val);
 extern enum cross_act check_cross (Coord);
 extern int toggle_cross (Coord);
 extern int dist_to_capital (int, Coord);
 extern char need_update (void);
+extern void select_next (void);
+extern void clear_crosses (void);
 
 #endif /* map.h  */
