@@ -26,6 +26,8 @@
 #define EVENT_ENCIRCLEMENT 8
 #define EVENT_NEWTURN 16
 #define EVENT_UNOCCUPY 32
+#define EVENT_DEFEAT 64
+#define EVENT_REFRESH 128
 
 typedef struct
 {
