@@ -47,18 +47,22 @@ alloc_err_str (err_no, file_name)
     "can't find map file: %s",
     "invalid map file: %s",
     "can't open display: %s\ntry motti -n for text mode",
+    "can't load font: %s",
     /* Fix me.  */
     "malloc error",
     "can't find color: %s",
-    "can't allocate a color: %s"
+    "can't allocate a color: %s",
+    "invalid mouse action: %s"
   };
+#if 0
   const int msg_len[] =
   {
-    22, 19, 48, 13, 19, 25
+    22, 19, 48, 18, 13, 19, 25, 23
   };
-    
-  str = (char *) my_malloc ((msg_len[err_no] + file_name == NULL ?
-			     strlen (file_name) : 0) * sizeof (char));
+#endif
+  str = (char *) my_malloc ((sizeof (msg[err_no]) + file_name == NULL
+			     ? strlen (file_name) : 0) * sizeof
+			    (char));
   sprintf (str, msg[err_no], file_name);
   return str;
 }
