@@ -55,19 +55,22 @@ XrmDatabase DB = {0};
 XColor *player_col;
 XColor *occupied_col;
 XColor misc_col[last_col_num];
-int players;
 
 Window main_win;
 Window but_win[NBUTS];
+Window turn_win;
 Window map_win;
 
-GC mapgc, capitalgc, crossgc;
+GC mapgc, capitalgc, crossgc, turngc;
 Pixmap att_pix[6], def_pix, gue_pix;
 Pixmap d_att_pix, d_def_pix, d_gue_pix;
 
+Coord turnwin_num_base[CROSS_MAX+2] = {0};
+
 static Visual *visual;
 static GC tempgc;
 
+static enum mouse_action parse_mouse_string (const char *);
 static void init_DB_tab (void);
 static int alloc_Xrm_color (const char *, const char *, const char *,
 			    XColor *);
@@ -76,8 +79,23 @@ static void make_att_buttons (void);
 static void make_att_pixs (void);
 static void make_def_pixs (void);
 static void make_gue_pixs (void);
+static void calculate_extents (const Font);
 static void alloc_gcs (void);
-static void set_hints (int, int, char **, int);
+static void set_hints (const int, const int, const int, char **, int);
+
+static enum mouse_action
+parse_mouse_string (act_name)
+     const char *act_name;
+{
+  register enum mouse_action i;
+  const char *all_acts[] = {
+    "mark", "select", "act", "last_place", "attack", "defend",
+    "guerilla", "reset", "none"
+  };
+  for (i = mark; i <= no_act; i++)
+    if (!strcasecmp (all_acts[i], act_name))
+      return i;
+}
 
 static void
 init_DB_tab ()
@@ -86,9 +104,7 @@ init_DB_tab ()
 
   enum types 
   {
-    type_short,
-    type_string,
-    end
+    type_short, type_string, type_mouse, type_geometry, end
   };
   struct val_tab
   {
@@ -99,6 +115,7 @@ init_DB_tab ()
     {
       const int val;
       const char *str;
+      const enum mouse_action mouse;
     } def;
   };
   struct val_tab vals[] =
@@ -107,12 +124,36 @@ init_DB_tab ()
      "Motti.Button.Width", 80},
     {type_short, &db_val_tab.but_height, "motti.button.height",
      "Motti.Button.Height", 20},
-    {type_string, &db_val_tab.geom_str, "motti.geometry",
+    {type_geometry, &db_val_tab.geom_str, "motti.geometry",
      "Motti.Geometry", 0},
+    {type_short, &db_val_tab.turnwin_width, "motti.turnwindow.width",
+     "Motti.Turnwindow.Width", 40},
+    {type_short, &db_val_tab.turnwin_height,
+     "motti.turnwindow.height", "Motti.Turnwindow.Height", 20},
+    {type_string, &db_val_tab.cross_left_font,
+     "motti.turnwindow.font", "Motti.Font", 0},
+    {type_short, &db_val_tab.map_square_size, "motti.map.squaresize",
+     "Motti.Map.Squaresize", 20},
+    {type_short, &db_val_tab.x_margin, "motti.x.margin",
+     "Motti.Margin", 10},
+    {type_short, &db_val_tab.y_margin, "motti.y.margin",
+     "Motti.Margin", 10},
+    {type_short, &db_val_tab.sel_width, "motti.selected.width",
+     "Motti.Width", 3},
+    {type_mouse, &db_val_tab.mouse[0], "motti.mousebutton1",
+     "Motti.Mousebutton1", mark},
+    {type_mouse, &db_val_tab.mouse[1], "motti.mousebutton2",
+     "Motti.Mousebutton2", sel},
+    {type_mouse, &db_val_tab.mouse[2], "motti.mousebutton3",
+     "Motti.Mousebutton3", act},
+    {type_mouse, &db_val_tab.mouse[3], "motti.mousebutton4",
+     "Motti.Mousebutton4", no_act},
+    {type_mouse, &db_val_tab.mouse[4], "motti.mousebutton5",
+     "Motti.Mousebutton5", no_act},
     {end, NULL, "", "", 0}
   };
   struct val_tab *val_ptr = vals;
-  vals[2].def.str = "1x1";
+  vals[5].def.str = "-adobe-helvetica-medium-r-*-*-20-*-*-*-*-*-*-*";
 
   while (val_ptr->type != end)
     {
@@ -133,7 +174,12 @@ init_DB_tab ()
 	      strcpy (str, value.addr);
 	      *((char **) val_ptr->ptr) = str;
 	      break;
-	    case end:
+	    case type_mouse:
+	      *((enum mouse_action *) val_ptr->ptr) =
+		parse_mouse_string (value.addr);
+	      if (*((enum mouse_action *) val_ptr->ptr) > no_act)
+		die (alloc_err_str (ERR_MOUSE, value.addr));
+	      break;
 	    }
 	}
       else
@@ -142,17 +188,18 @@ init_DB_tab ()
 	    {
 	    case type_short:
 	      *((short *) val_ptr->ptr) = (short) val_ptr->def.val;
-	      goto hardcoded;
 	      break;
 	    case type_string:
+	    case type_geometry:
 	      *((const char **) val_ptr->ptr) = val_ptr->def.str;
-	      goto hardcoded;
 	      break;
-	    case end:
+	    case type_mouse:
+	      *((enum mouse_action *) val_ptr->ptr) =
+		val_ptr->def.mouse;
 	      break;
-	    hardcoded:
-	      fprintf (stderr, "motti: using hard-coded default resource value for %s\n", val_ptr->name);
 	    }
+	  if (val_ptr->type != type_geometry)
+	    fprintf (stderr, "motti: using hard-coded default resource value for %s\n", val_ptr->name);
 	}
       val_ptr++;
     }
@@ -247,7 +294,7 @@ alloc_Xrm_color (instance, class, deflt, color)
       if (deflt)
 	{
 	  color_name = deflt;
-	  fprintf (stderr, "motti: using hard-coded default color %s for %s\n",
+	  fprintf (stderr, "motti: using hard-coded default color \"%s\" for %s\n",
 		   deflt, instance);
 	}
       else
@@ -278,20 +325,8 @@ alloc_colors ()
   if (i < StaticColor)
     return 0;
 
-  if (XrmGetResource (DB, "motti.players", "Motti.Players",
-		      &str_type, &value))
-    {
-      players = atoi (value.addr);
-      if (players > MAXPLAYERS)
-	die ("too many players");
-      else if(players < 2)
-	die ("too few players");
-    }
-  else
-    players = 2;
-
-  player_col = (XColor *) my_malloc (sizeof(XColor)*players);
-  occupied_col = (XColor *) my_malloc (sizeof(XColor)*players);
+  player_col = (XColor *) my_malloc (sizeof(XColor)*game_map.players);
+  occupied_col = (XColor *) my_malloc (sizeof(XColor)*game_map.players);
 
   if (!(alloc_Xrm_color ("motti.background", "Motti.Background",
 			"white", &misc_col[bg_col])
@@ -311,16 +346,24 @@ alloc_colors ()
 			    &misc_col[but_fg])
 	&& alloc_Xrm_color ("motti.button.background",
 			    "Motti.Background", "white",
-			    &misc_col[but_bg])))
+			    &misc_col[but_bg])
+	&& alloc_Xrm_color ("motti.color.cross_left",
+			    "Motti.Foreground", "black",
+			    &misc_col[cross_left])
+	&& alloc_Xrm_color ("motti.color.turnwin_border",
+			    "Motti.Background", "black",
+			    &misc_col[turn_win_border])))
     return 0;
 
-#define MAXDEFCOLPLAYER 2
-  for (i = 1; i <= players; i++)
+#define MAXDEFCOLPLAYER 4
+  for (i = 1; i <= game_map.players; i++)
     {
       const char *def_col[] =
       {
-	"RGBi:0.4/0/0", "RGBi:1.0/0/0",
-	"RGBi:0/0/0.4", "RGBi:0/0/1.0",
+	"DarkRed", "Red",
+	"DarkBlue", "Blue",
+	"DarkGreen", "Green",
+	"DarkGoldenrod4", "DarkGoldenrod1",
 	""
       };
       char instancename[25];
@@ -543,19 +586,41 @@ make_gue_pixs ()
   XFreePixmap (display, pix);
 }
 
+static void
+calculate_extents (font)
+     const Font font;
+{
+  register int i;
+  for (i = 0; i <= CROSS_MAX+1; i++)
+    {
+      int null;
+      const char *nums[] = {"6", "5", "4", "3", "2", "1", "0", "*"};
+      XCharStruct charstruct;
+      XQueryTextExtents (display, font, nums[i], 1, &null, &null,
+			 &null, &charstruct);
+      turnwin_num_base[i].x = db_val_tab.turnwin_width/2 -
+	(charstruct.width-charstruct.lbearing)/2;
+      turnwin_num_base[i].y = db_val_tab.turnwin_height/2 -
+	(charstruct.ascent+charstruct.descent)/2 + charstruct.ascent;
+    }      
+}
+
 static void
 alloc_gcs ()
 {
   XGCValues gcval = {0};
   Pixmap capital_stipple, cross_stipple;
 
-  capital_stipple = XCreatePixmap (display, main_win, MAPSQUARESIZE,
-				   MAPSQUARESIZE, 1);
-  cross_stipple = XCreatePixmap (display, main_win, MAPSQUARESIZE,
-				 MAPSQUARESIZE, 1);
+  capital_stipple = XCreatePixmap (display, main_win,
+				   db_val_tab.map_square_size,
+				   db_val_tab.map_square_size, 1);
+  cross_stipple = XCreatePixmap (display, main_win,
+				 db_val_tab.map_square_size,
+				 db_val_tab.map_square_size, 1);
 
-  make_map_stipples (capital_stipple, cross_stipple, MAPSQUARESIZE,
-		     MAPSQUARESIZE);
+  make_map_stipples (capital_stipple, cross_stipple,
+		     db_val_tab.map_square_size,
+		     db_val_tab.map_square_size);
 
   /* These three are set to draw capitals. */
   gcval.foreground = misc_col[capital_col].pixel;
@@ -573,14 +638,26 @@ alloc_gcs ()
    * time */
   /* Note to myself: change this to make BW window. */
   mapgc = XCreateGC (display, main_win, 0, &gcval);
+
+  /* This gc is used to draw the amount of crosses left to the turn
+     window.  */
+  gcval.foreground = misc_col[cross_left].pixel;
+  gcval.background = occupied_col[game_map.turn-1].pixel;
+  gcval.font = XLoadFont (display, db_val_tab.cross_left_font);
+
+  if (!gcval.font)
+    die (alloc_err_str (ERR_NOFONT, db_val_tab.cross_left_font));
+  calculate_extents (gcval.font);
+  turngc = XCreateGC (display, main_win, GCBackground | GCForeground |
+		      GCFont, &gcval);
 }
 
 int but_width[3];
 int but_height;
 
 static void
-set_hints (min_width, min_height, argv, argc)
-     int min_width, min_height;
+set_hints (min_width, min_height, geom_specified, argv, argc)
+     const int min_width, min_height, geom_specified;
      char **argv;
      int argc;
 {
@@ -601,13 +678,15 @@ set_hints (min_width, min_height, argv, argc)
   wm_hints = XAllocWMHints ();
   class_hints = XAllocClassHint ();
 
-  /* TODO: IIRC here is something wrong here, not sure what.  */
-  size_hints->flags = PPosition | PSize | PMinSize;
+  if (geom_specified)
+    size_hints->flags = PPosition | PSize | PMinSize;
+  else
+    size_hints->flags = PSize | PMinSize;
   size_hints->min_width = min_width;
   size_hints->min_height = min_height;
 
   wm_hints->initial_state = NormalState;
-  wm_hints->input = True;
+  wm_hints->input = False;
 
   /* TODO: IIRC these shouldn't be hardcoded.  */
   class_hints->res_name = "motti";
@@ -632,19 +711,14 @@ open_windows (argc, argv)
      char **argv;
 {
   XSetWindowAttributes win_attribs;
-  int x = 0, y = 0, width, height;
+  int x = 0, y = 0, width = 1, height = 1, geom_specified = 1;
   char *str_type;
   XrmValue value;
 
-  if (!db_val_tab.geom_str)
-    {
-      XrmValue value;
-      char *str_type;
-      XrmGetResource (DB, "Motti.Geometry", "motti.geometry",
-		      &str_type, &value);
-      db_val_tab.geom_str = value.addr;
-    }
-  XParseGeometry (db_val_tab.geom_str, &x, &y, &width, &height);
+  if (db_val_tab.geom_str)
+    XParseGeometry (db_val_tab.geom_str, &x, &y, &width, &height);
+  else
+    geom_specified = 0;
 
   win_attribs.background_pixel = misc_col[bg_col].pixel;
   main_win = XCreateWindow (display, RootWindow(display, screen_num),
@@ -660,7 +734,9 @@ open_windows (argc, argv)
   XFreeGC (display, tempgc);
 
   win_attribs.border_pixel = misc_col[but_fg].pixel;
-  but_win[ATT] = XCreateWindow (display, main_win, X_MARGIN, Y_MARGIN,
+  but_win[ATT] = XCreateWindow (display, main_win,
+				db_val_tab.x_margin,
+				db_val_tab.y_margin,
 				db_val_tab.but_width,
 				db_val_tab.but_height, 1,
 				CopyFromParent, InputOutput,
@@ -668,35 +744,54 @@ open_windows (argc, argv)
 				&win_attribs);
 
   but_win[DEF] = XCreateWindow (display, main_win,
-				db_val_tab.but_width + X_MARGIN*2,
-				Y_MARGIN, db_val_tab.but_width,
+				db_val_tab.but_width +
+				db_val_tab.x_margin*2,
+				db_val_tab.y_margin,
+				db_val_tab.but_width,
 				db_val_tab.but_height, 1,
 				CopyFromParent, InputOutput,
 				CopyFromParent, CWBorderPixel,
 				&win_attribs);
 
   but_win[GUE] = XCreateWindow (display, main_win,
-				db_val_tab.but_width*2 + X_MARGIN*3,
-				Y_MARGIN, db_val_tab.but_width,
+				db_val_tab.but_width*2 +
+				db_val_tab.x_margin*3,
+				db_val_tab.y_margin,
+				db_val_tab.but_width,
 				db_val_tab.but_height, 1,
 				CopyFromParent, InputOutput,
 				CopyFromParent, CWBorderPixel,
 				&win_attribs);
 
-  x = X_MARGIN;
+  turn_win = XCreateSimpleWindow (display, main_win,
+				  db_val_tab.but_width*3 +
+				  db_val_tab.x_margin*4,
+				  db_val_tab.y_margin,
+				  db_val_tab.turnwin_width,
+				  db_val_tab.turnwin_height, 1,
+				  misc_col[turn_win_border].pixel,
+				  player_col[game_map.turn-1].pixel);
+
+  x = db_val_tab.x_margin;
   /* There must be room for action buttons above the map.  */
-  y = Y_MARGIN*2 + db_val_tab.but_height;
-  /* TODO: Scrollable and resizeable map window.  */
+  y = db_val_tab.y_margin*2 + (db_val_tab.but_height >
+			       db_val_tab.turnwin_height
+			       ? db_val_tab.but_height :
+			       db_val_tab.turnwin_height);
+  /* TODO: Scrollable and resizeable map window.  Propably too much
+     trouble anyway, at least with bare Xlib!  */
   win_attribs.border_pixel = misc_col[fg_col].pixel;
   win_attribs.background_pixel = misc_col[map_bg_col].pixel;
   map_win = XCreateWindow (display, main_win, x, y, game_map.width *
-			   MAPSQUARESIZE, game_map.height *
-			   MAPSQUARESIZE, 2, CopyFromParent,
+			   db_val_tab.map_square_size, game_map.height *
+			   db_val_tab.map_square_size, 2, CopyFromParent,
 			   InputOutput, CopyFromParent, CWBorderPixel
 			   | CWBackPixel, &win_attribs);
 
-  set_hints (x + game_map.width * MAPSQUARESIZE + 2 * X_MARGIN, y +
-	     game_map.height * MAPSQUARESIZE + Y_MARGIN, argv, argc);
+  set_hints (x + game_map.width * db_val_tab.map_square_size + 2 *
+	     db_val_tab.x_margin, y + game_map.height *
+	     db_val_tab.map_square_size + db_val_tab.y_margin,
+	     geom_specified, argv, argc);
 }
 
 #include "bitmaps/map_pointer_active"
