@@ -19,20 +19,34 @@
 
 #define _XINIT_H
 
+#include "map.h"
+
 enum misc_colors {
   capital_col, cross_col,
   fg_col, bg_col,
   map_bg_col,
   but_fg, but_bg,
+  cross_left, turn_win_border,
   last_col_num
 };
 
+enum mouse_action {
+  mark, sel, act, last_place, attack, defend, guerilla, reset, no_act
+};
+
 struct db_vals {
   char *geom_str;
   short but_width, but_height;
+  short turnwin_width, turnwin_height;
+  char *cross_left_font;
+  short map_square_size;
+  short x_margin, y_margin;
+  enum mouse_action mouse[5];
+  short sel_width;
 };
 
 extern struct db_vals db_val_tab;
+extern Coord turnwin_num_base[];
 
 extern int open_display (char *);
 extern void create_DB (void);
