@@ -21,15 +21,6 @@
 
 #define NBUTS 3
 
-/* Note to myself: put these in app-defaults.  */
-#define Y_MARGIN 10
-#define X_MARGIN 10
-#define MAPSQUARESIZE 20
-#define MINMAPHEIGHT 10
-
-#define BUT_WIDTH 80
-#define BUT_HEIGHT 20
-
 #include "map.h"
 #include "occupy.h"
 
@@ -37,7 +28,8 @@ enum button_class {
   ATT, DEF, GUE
 };
 
-extern void update_buttons (void);
+extern void update_buttons (const int);
+extern void update_cross_win (int);
 extern void draw_map (int, int, int, int);
 extern void draw_effects (Action);
 extern Coord win_pos2map_coord (int, int);
